/*
 * Thermo.c
 *
 * Created: 09.04.2018 0:16:04
 *  Author: Evgeniy
 */ 

#define F_CPU 4800000UL

#include <avr/io.h>
#include <avr/eeprom.h>
#include <util/delay.h>
#include <avr/interrupt.h>

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  ,    
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#define DR 0
#define DG 3
#define DB 2

#define BT 1

#define DS 4

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//   .     :    ,   eeprom  
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#define THMIN 20
#define THMAX 30
#define PERIOD 10
#define ACTIVE 3
#define TRIG 0

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

unsigned char period, active, trig, allow_led, allow_ds18b20;
int thint, thmin, thmax;
volatile unsigned char counter = 0;
volatile unsigned int tis = 0;

ISR(TIM0_OVF_vect)
{
	if (counter == 18)	// . 4.8/1024/256 = 1/18. (18   )
	{
		counter = 0;
		tis++;
	}
	counter++;
	
	if ((allow_ds18b20 != 1) && (allow_led != 1) && (period == tis))		//       
	{																		/// ,  
		allow_ds18b20 = 1;
	}
	else if ((allow_ds18b20 == 2) && (allow_led == 1) && (active == tis))	//   
	{
		if (trig == 1)														//  "  "  
		{
			TCCR0B = 0x00;
		}
		
		allow_led = 0;														// ,  
		allow_ds18b20 = 0;
		tis = 0;
	}
}

ISR(INT0_vect)																//  
{																			
	switch(trig)
	{
		case 1:
			tis = 0;
			allow_ds18b20 = 1;
			TCCR0B = 0x05;
			break;
		case 2:
			if ((allow_ds18b20 != 1) && (allow_led != 1))					//   "   ", 
			{																//   ,   
				tis = 0;													/// 
				allow_ds18b20 = 1;
			}
			break;
	}
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//    1-Wire
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void ow_reset(void)
{
	PORTB &= ~(1 << DS);
	DDRB |= (1 << DS);
	_delay_us(480);
	DDRB &= ~(1 << DS);
	_delay_us(480);
}

void ow_write(unsigned char dat)
{
	unsigned char i;
	for(i = 0;i < 8;i++)
	{
		DDRB |= (1 << DS);
		_delay_us(10);
		if(dat & 0x01)
		{
			DDRB &= ~(1 << DS);
			_delay_us(65);
		}
		else
		{
			_delay_us(55);
		}
		dat = dat >> 1;
		DDRB &= ~(1 << DS);
		_delay_us(5);
	}
}

unsigned char ow_read(void)
{
	unsigned char i;
	unsigned char dat = 0;
	for(i = 0;i < 8;i++)
	{
		DDRB |= (1 << DS);
		_delay_us(3);
		DDRB &= ~(1 << DS);
		_delay_us(10);
		dat = dat >> 1;
		if(PINB & (1 << DS))
		{
			dat |= 0x80;
		}
		_delay_us(53);
	}
	return dat;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void init(void)
{
	PORTB &= (1 << DR)|(1 << DG)|(1 << DB);
	PORTB |= (1 << BT);
	DDRB |= (1 << DR)|(1 << DG)|(1 << DB);
	
	TCCR0A = 0x00;
	TCCR0B = 0x05;
	TIMSK0 = 0x02;
	
	MCUCR = 0x03;
	GIMSK = 0x40;
	
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//  EEPROM
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
	unsigned char eidata = 0;
	eidata = eeprom_read_byte((unsigned char*)0);
	if (eidata != 18)
	{
		eeprom_write_byte((unsigned char*)0, 18);		// 
		if (THMIN < 0)
		{
			unsigned char a = THMIN * -1;
			eeprom_write_byte((unsigned char*)1, 127);
			eeprom_write_byte((unsigned char*)2, a);//   .    - ,       - .
		}
		else
		{
			eeprom_write_byte((unsigned char*)1, 0);
			eeprom_write_byte((unsigned char*)2, THMIN);
		}
		if (THMAX < 0)
		{
			unsigned char a = THMAX * -1;
			eeprom_write_byte((unsigned char*)3, 127);
			eeprom_write_byte((unsigned char*)4, a);// ,   - 
		}
		else
		{
			eeprom_write_byte((unsigned char*)3, 0);
			eeprom_write_byte((unsigned char*)4, THMAX);
		}
		eeprom_write_byte((unsigned char*)5, TRIG);		// "0" -  ,  "1" - ,  "2" -    ,  "3" - . .
		eeprom_write_byte((unsigned char*)6, PERIOD);	// , . (  5 .)
		eeprom_write_byte((unsigned char*)7, ACTIVE);	// , .
	}
	
	unsigned char a = eeprom_read_byte((unsigned char*)1);
	if (a == 127)
	{
		a = eeprom_read_byte((unsigned char*)2);
		thmin = a * -1;
	}
	else
	{
		thmin = eeprom_read_byte((unsigned char*)2);
	}
	unsigned char b = eeprom_read_byte((unsigned char*)3);
	if (b == 127)
	{
		b = eeprom_read_byte((unsigned char*)4);
		thmax = b * -1;
	}
	else
	{
		thmax = eeprom_read_byte((unsigned char*)4);
	}
	trig = eeprom_read_byte((unsigned char*)5);
	period = eeprom_read_byte((unsigned char*)6);
	if (period < 5)
	{
		period = 5;
	}
	active = eeprom_read_byte((unsigned char*)7);
	
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
	switch(trig)
	{
		case 0:
			GIMSK = 0x00;
			break;
		case 1:
			TCCR0B = 0x00;
			break;
		case 3:
			TCCR0B = 0x00;
			GIMSK = 0x00;
			
			allow_ds18b20 = 1;
			allow_led = 1;
			break;
	}
	
	sei();
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int main(void)
{
	init();
	while(1)
	{
		if (allow_ds18b20 == 1)
		{
			
			_delay_ms(100);
			ow_reset();				//  DS18B20
			ow_write(0xCC);			//  ROM  
			ow_write(0x44);			//   
			ow_reset();				//  DS18B20
			ow_write(0xCC);			//  ROM  
			ow_write(0xBE);			//   
			_delay_ms(750);
			
			unsigned char thchar[2];
			thchar[0] = ow_read();	//LSB ::   2  
			thchar[1] = ow_read();	//MSB
			thint = ((thchar[1] << 8) + thchar[0]) >> 4;
			if (thint > 1000)		//   
			{
				thint = -(4096 - thint);
			}
			
			if (trig != 3)
			{
				allow_ds18b20 = 2;
				allow_led = 1;
			}
			
			tis = 0;
		}
		
		if (allow_led == 1)	// 
		{
			if (thint < thmin)
			{
				PORTB |= (1 << DR)|(1 << DG);
				PORTB &= ~(1 << DB);
			}
			else if (thint < thmax)
			{
				PORTB |= (1 << DR)|(1 << DB);
				PORTB &= ~(1 << DG);
			}
			else
			{
				PORTB |= (1 << DG)|(1 << DB);
				PORTB &= ~(1 << DR);
			}
		}
		else
		{
			PORTB |= (1 << DR)|(1 << DG)|(1 << DB);
		}
	}
}